@@ -7,20 +7,28 @@
 #ifndef KATE_TEXTBLOCK_H
 #define KATE_TEXTBLOCK_H
 
+#include <unordered_map>
 #include <unordered_set>
 
 #include <QSet>
+#include <QVarLengthArray>
 #include <QVector>
 
-#include "katetextline.h"
 #include <ktexteditor/cursor.h>
 #include <ktexteditor_export.h>
 
+namespace KTextEditor
+{
+class View;
+}
+
 namespace Kate
 {
 class TextBuffer;
 class TextCursor;
 class TextRange;
+class TextLineData;
+typedef QSharedPointer<TextLineData> TextLine;
 
 /**
  * Class representing a text block.
@@ -81,7 +89,7 @@ public:
      */
     int lines() const
     {
-        return m_lines.size();
+        return static_cast<int>(m_lines.size());
     }
 
     /**
@@ -155,13 +163,12 @@ public:
 
     /**
      * Return all ranges in this block which might intersect the given line.
-     * @param line line to check intersection
-     * @return list of sets of possible candidate ranges
+     * @param line                          line to check intersection
+     * @param view                          only return ranges associated with given view
+     * @param rangesWithAttributeOnly       ranges with attributes only?
+     * @return list of possible candidate ranges
      */
-    QList<QSet<TextRange *>> rangesForLine(int line) const
-    {
-        return QList<QSet<TextRange *>>() << m_uncachedRanges << cachedRangesForLine(line);
-    }
+    QVector<TextRange *> rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const;
 
     /**
      * Is the given range contained in this block?
@@ -170,7 +177,7 @@ public:
      */
     bool containsRange(TextRange *range) const
     {
-        return m_cachedLineForRanges.contains(range) || m_uncachedRanges.contains(range);
+        return m_cachedLineForRanges.find(range) != m_cachedLineForRanges.end() || m_uncachedRanges.contains(range);
     }
 
     /**
@@ -218,7 +225,7 @@ public:
     QSet<TextRange *> cachedRangesForLine(int line) const
     {
         line -= m_startLine;
-        if (line >= 0 && line < m_cachedRangesForLine.size()) {
+        if (line >= 0 && (size_t)line < m_cachedRangesForLine.size()) {
             return m_cachedRangesForLine[line];
         } else {
             return QSet<TextRange *>();
@@ -252,17 +259,17 @@ private:
      * Contains for each line-offset the ranges that were cached into it.
      * These ranges are fully contained by the line.
      */
-    QVector<QSet<TextRange *>> m_cachedRangesForLine;
+    std::vector<QSet<TextRange *>> m_cachedRangesForLine;
 
     /**
      * Maps for each cached range the line into which the range was cached.
      */
-    QHash<TextRange *, int> m_cachedLineForRanges;
+    std::unordered_map<TextRange *, int> m_cachedLineForRanges;
 
     /**
      * This contains all the ranges that are not cached.
      */
-    QSet<TextRange *> m_uncachedRanges;
+    QVarLengthArray<TextRange *, 1> m_uncachedRanges;
 };
 
 }
