@@ -1,24 +1,12 @@
-/* This file is part of the KDE libraries
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/*
+    SPDX-FileCopyrightText: KDE Developers
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "marks.h"
-#include "kateview.h"
 #include "katedocument.h"
+#include "kateview.h"
 #include <vimode/inputmodemanager.h>
 #include <vimode/modes/normalvimode.h>
 
@@ -26,24 +14,22 @@
 
 using namespace KateVi;
 
-namespace {
-    const QChar BeginEditYanked = QLatin1Char('[');
-    const QChar EndEditYanked = QLatin1Char(']');
-    const QChar LastChange = QLatin1Char('.');
-    const QChar InsertStopped = QLatin1Char('^');
-    const QChar SelectionBegin = QLatin1Char('<');
-    const QChar SelectionEnd = QLatin1Char('>');
-    const QChar FirstUserMark = QLatin1Char('a');
-    const QChar LastUserMark = QLatin1Char('z');
-    const QChar BeforeJump = QLatin1Char('\'');
-    const QChar BeforeJumpAlter = QLatin1Char('`');
-    const QChar UserMarks[] = {QLatin1Char('a'), QLatin1Char('b'), QLatin1Char('c'), QLatin1Char('d'),
-                               QLatin1Char('e'), QLatin1Char('f'), QLatin1Char('g'), QLatin1Char('h'),
-                               QLatin1Char('i'), QLatin1Char('j'), QLatin1Char('k'), QLatin1Char('l'),
-                               QLatin1Char('m'), QLatin1Char('n'), QLatin1Char('o'), QLatin1Char('p'),
-                               QLatin1Char('q'), QLatin1Char('r'), QLatin1Char('s'), QLatin1Char('t'),
-                               QLatin1Char('u'), QLatin1Char('v'), QLatin1Char('w'), QLatin1Char('x'),
-                               QLatin1Char('y'), QLatin1Char('z')};
+namespace
+{
+const QChar BeginEditYanked = QLatin1Char('[');
+const QChar EndEditYanked = QLatin1Char(']');
+const QChar LastChange = QLatin1Char('.');
+const QChar InsertStopped = QLatin1Char('^');
+const QChar SelectionBegin = QLatin1Char('<');
+const QChar SelectionEnd = QLatin1Char('>');
+const QChar FirstUserMark = QLatin1Char('a');
+const QChar LastUserMark = QLatin1Char('z');
+const QChar BeforeJump = QLatin1Char('\'');
+const QChar BeforeJumpAlter = QLatin1Char('`');
+const QChar UserMarks[] = {QLatin1Char('a'), QLatin1Char('b'), QLatin1Char('c'), QLatin1Char('d'), QLatin1Char('e'), QLatin1Char('f'), QLatin1Char('g'),
+                           QLatin1Char('h'), QLatin1Char('i'), QLatin1Char('j'), QLatin1Char('k'), QLatin1Char('l'), QLatin1Char('m'), QLatin1Char('n'),
+                           QLatin1Char('o'), QLatin1Char('p'), QLatin1Char('q'), QLatin1Char('r'), QLatin1Char('s'), QLatin1Char('t'), QLatin1Char('u'),
+                           QLatin1Char('v'), QLatin1Char('w'), QLatin1Char('x'), QLatin1Char('y'), QLatin1Char('z')};
 }
 
 Marks::Marks(InputModeManager *imm)
@@ -54,10 +40,6 @@ Marks::Marks(InputModeManager *imm)
     connect(m_doc, &KTextEditor::DocumentPrivate::markChanged, this, &Marks::markChanged);
 }
 
-Marks::~Marks()
-{
-}
-
 void Marks::readSessionConfig(const KConfigGroup &config)
 {
     QStringList marks = config.readEntry("ViMarks", QStringList());
@@ -72,51 +54,59 @@ void Marks::readSessionConfig(const KConfigGroup &config)
 void Marks::writeSessionConfig(KConfigGroup &config) const
 {
     QStringList l;
-    Q_FOREACH (QChar key, m_marks.keys()) {
-        l << key << QString::number(m_marks.value(key)->line())
-        << QString::number(m_marks.value(key)->column());
+    const auto keys = m_marks.keys();
+    for (QChar key : keys) {
+        l << key << QString::number(m_marks.value(key)->line()) << QString::number(m_marks.value(key)->column());
     }
     config.writeEntry("ViMarks", l);
 }
 
-void Marks::setMark(const QChar &_mark, const KTextEditor::Cursor &pos, const bool moveoninsert)
+void Marks::setMark(const QChar &_mark, const KTextEditor::Cursor &pos)
 {
+    // move on insert is type based, this allows to reuse cursors!
+    // reuse is important for editing intensive things like replace-all
+    const bool moveoninsert = _mark != BeginEditYanked;
+
     m_settingMark = true;
-    uint marktype = m_doc->mark(pos.line());
 
     // ` and ' is the same register (position before jump)
     const QChar mark = (_mark == BeforeJumpAlter) ? BeforeJump : _mark;
 
-    // delete old cursor if any
+    // if we have already a cursor for this type: adjust it
+    bool needToAdjustVisibleMark = true;
     if (KTextEditor::MovingCursor *oldCursor = m_marks.value(mark)) {
-
-        int  number_of_marks = 0;
-
-        foreach (QChar c, m_marks.keys()) {
-            if (m_marks.value(c)->line() ==  oldCursor->line()) {
-                number_of_marks++;
+        // cleanup mark display only if line changes
+        needToAdjustVisibleMark = oldCursor->line() != pos.line();
+        if (needToAdjustVisibleMark) {
+            int number_of_marks = 0;
+            const auto keys = m_marks.keys();
+            for (QChar c : keys) {
+                if (m_marks.value(c)->line() == oldCursor->line()) {
+                    number_of_marks++;
+                }
+            }
+            if (number_of_marks == 1) {
+                m_doc->removeMark(oldCursor->line(), KTextEditor::MarkInterface::markType01);
             }
         }
 
-        if (number_of_marks == 1 && pos.line() != oldCursor->line()) {
-            m_doc->removeMark(oldCursor->line(), KTextEditor::MarkInterface::markType01);
-        }
-
-        delete oldCursor;
+        // adjust position
+        oldCursor->setPosition(pos);
+    } else {
+        // if no old mark of that type, create new one
+        const KTextEditor::MovingCursor::InsertBehavior behavior =
+            moveoninsert ? KTextEditor::MovingCursor::MoveOnInsert : KTextEditor::MovingCursor::StayOnInsert;
+        m_marks.insert(mark, m_doc->newMovingCursor(pos, behavior));
     }
 
-    KTextEditor::MovingCursor::InsertBehavior behavior = moveoninsert ? KTextEditor::MovingCursor::MoveOnInsert : KTextEditor::MovingCursor::StayOnInsert;
-    // create and remember new one
-    m_marks.insert(mark, m_doc->newMovingCursor(pos, behavior));
-
-    // Showing what mark we set:
+    // Showing what mark we set, can be skipped if we did not change the line
     if (isShowable(mark)) {
-        if (!(marktype & KTextEditor::MarkInterface::markType01)) {
+        if (needToAdjustVisibleMark && !(m_doc->mark(pos.line()) & KTextEditor::MarkInterface::markType01)) {
             m_doc->addMark(pos.line(), KTextEditor::MarkInterface::markType01);
         }
 
         // only show message for active view
-        if ( m_inputModeManager->view()->viewInputMode() == KTextEditor::View::ViInputMode ) {
+        if (m_inputModeManager->view()->viewInputMode() == KTextEditor::View::ViInputMode) {
             if (m_doc->activeView() == m_inputModeManager->view()) {
                 m_inputModeManager->getViNormalMode()->message(i18n("Mark set: %1", mark));
             }
@@ -136,11 +126,8 @@ KTextEditor::Cursor Marks::getMarkPosition(const QChar &mark) const
     return KTextEditor::Cursor::invalid();
 }
 
-void Marks::markChanged(KTextEditor::Document *doc,
-                        KTextEditor::Mark mark,
-                        KTextEditor::MarkInterface::MarkChangeAction action)
+void Marks::markChanged(KTextEditor::Document *doc, KTextEditor::Mark mark, KTextEditor::MarkInterface::MarkChangeAction action)
 {
-
     Q_UNUSED(doc)
 
     if (mark.type != KTextEditor::MarkInterface::Bookmark || m_settingMark) {
@@ -148,7 +135,8 @@ void Marks::markChanged(KTextEditor::Document *doc,
     }
 
     if (action == KTextEditor::MarkInterface::MarkRemoved) {
-        foreach (QChar markerChar, m_marks.keys()) {
+        const auto keys = m_marks.keys();
+        for (QChar markerChar : keys) {
             if (m_marks.value(markerChar)->line() == mark.line) {
                 m_marks.remove(markerChar);
             }
@@ -156,7 +144,7 @@ void Marks::markChanged(KTextEditor::Document *doc,
     } else if (action == KTextEditor::MarkInterface::MarkAdded) {
         bool freeMarkerCharFound = false;
 
-        for (const QChar markerChar : UserMarks) {
+        for (const QChar &markerChar : UserMarks) {
             if (!m_marks.value(markerChar)) {
                 setMark(markerChar, KTextEditor::Cursor(mark.line, 0));
                 freeMarkerCharFound = true;
@@ -164,7 +152,8 @@ void Marks::markChanged(KTextEditor::Document *doc,
             }
         }
 
-        if (!freeMarkerCharFound) {
+        // only show error when we are in Vi input mode
+        if (!freeMarkerCharFound && m_inputModeManager->view()->viewInputMode() == KTextEditor::View::ViInputMode) {
             m_inputModeManager->getViNormalMode()->error(i18n("There are no more chars for the next bookmark."));
         }
     }
@@ -181,7 +170,7 @@ void Marks::syncViMarksAndBookmarks()
         }
 
         bool thereIsViMarkForThisLine = false;
-        Q_FOREACH (const KTextEditor::MovingCursor *cursor, m_marks) {
+        for (auto cursor : qAsConst(m_marks)) {
             if (cursor->line() == mark->line) {
                 thereIsViMarkForThisLine = true;
                 break;
@@ -192,7 +181,7 @@ void Marks::syncViMarksAndBookmarks()
             continue;
         }
 
-        for (const QChar markerChar : UserMarks) {
+        for (const QChar &markerChar : UserMarks) {
             if (!m_marks.value(markerChar)) {
                 setMark(markerChar, KTextEditor::Cursor(mark->line, 0));
                 break;
@@ -201,7 +190,8 @@ void Marks::syncViMarksAndBookmarks()
     }
 
     // For showable vi mark a line should be bookmarked.
-    Q_FOREACH (const QChar &markChar, m_marks.keys()) {
+    const auto keys = m_marks.keys();
+    for (QChar markChar : keys) {
         if (!isShowable(markChar)) {
             continue;
         }
@@ -227,10 +217,10 @@ void Marks::syncViMarksAndBookmarks()
 QString Marks::getMarksOnTheLine(int line) const
 {
     QString res;
-
-    Q_FOREACH (QChar markerChar, m_marks.keys()) {
+    const auto keys = m_marks.keys();
+    for (QChar markerChar : keys) {
         if (m_marks.value(markerChar)->line() == line) {
-            res += markerChar + QLatin1String(":") + QString::number(m_marks.value(markerChar)->column()) + QLatin1String(" ");
+            res += markerChar + QLatin1Char(':') + QString::number(m_marks.value(markerChar)->column()) + QLatin1Char(' ');
         }
     }
 
@@ -244,7 +234,7 @@ bool Marks::isShowable(const QChar &mark)
 
 void Marks::setStartEditYanked(const KTextEditor::Cursor &pos)
 {
-    setMark(BeginEditYanked, pos, false);
+    setMark(BeginEditYanked, pos);
 }
 
 void Marks::setFinishEditYanked(const KTextEditor::Cursor &pos)
