@@ -1,49 +1,34 @@
-/*  This file is part of the KDE libraries and the Kate part.
- *
- *  Copyright (C) 2008 - 2012 Erlend Hamberg <ehamberg@gmail.com>
- *  Copyright (C) 2009 Paul Gideon Dann <pdgiddie@gmail.com>
- *  Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
- *  Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/*
+    SPDX-FileCopyrightText: 2008-2009 Erlend Hamberg <ehamberg@gmail.com>
+    SPDX-FileCopyrightText: 2009 Paul Gideon Dann <pdgiddie@gmail.com>
+    SPDX-FileCopyrightText: 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+    SPDX-FileCopyrightText: 2012-2013 Simon St James <kdedevel@etotheipiplusone.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
-#include <vimode/modes/modebase.h>
-#include <vimode/range.h>
-#include "kateglobal.h"
-#include "kateviinputmode.h"
-#include <vimode/globalstate.h>
-#include <vimode/modes/visualvimode.h>
-#include <vimode/modes/normalvimode.h>
-#include <vimode/modes/replacevimode.h>
-#include <vimode/inputmodemanager.h>
-#include "katelayoutcache.h"
 #include "kateconfig.h"
 #include "katedocument.h"
-#include "kateviewinternal.h"
+#include "kateglobal.h"
+#include "katelayoutcache.h"
 #include "katerenderer.h"
-#include <vimode/marks.h>
+#include "kateviewinternal.h"
+#include "kateviinputmode.h"
+#include <vimode/globalstate.h>
+#include <vimode/inputmodemanager.h>
 #include <vimode/jumps.h>
+#include <vimode/lastchangerecorder.h>
+#include <vimode/marks.h>
+#include <vimode/modes/modebase.h>
+#include <vimode/modes/normalvimode.h>
+#include <vimode/modes/replacevimode.h>
+#include <vimode/modes/visualvimode.h>
 #include <vimode/registers.h>
 #include <vimode/searcher.h>
-#include <vimode/lastchangerecorder.h>
 
-#include <QString>
 #include <KLocalizedString>
-#include <QRegExp>
+#include <QRegularExpression>
+#include <QString>
 
 using namespace KateVi;
 
@@ -57,13 +42,12 @@ using namespace KateVi;
 // HELPER METHODS
 ////////////////////////////////////////////////////////////////////////////////
 
-void ModeBase::yankToClipBoard(QChar chosen_register, QString text)
+void ModeBase::yankToClipBoard(QChar chosen_register, const QString &text)
 {
-    //only yank to the clipboard if no register was specified,
+    // only yank to the clipboard if no register was specified,
     // textlength > 1 and there is something else then whitespace
-    if ((chosen_register == QLatin1Char('0') || chosen_register == QLatin1Char('-'))
-        && text.length() > 1 && !text.trimmed().isEmpty())
-    {
+    if ((chosen_register == QLatin1Char('0') || chosen_register == QLatin1Char('-') || chosen_register == PrependNumberedRegister) && text.length() > 1
+        && !text.trimmed().isEmpty()) {
         KTextEditor::EditorPrivate::self()->copyToClipboard(text);
     }
 }
@@ -72,7 +56,7 @@ bool ModeBase::deleteRange(Range &r, OperationMode mode, bool addToRegister)
 {
     r.normalize();
     bool res = false;
-    QString removedText = getRange(r, mode);
+    const QString removedText = getRange(r, mode);
 
     if (mode == LineWise) {
         doc()->editStart();
@@ -84,14 +68,22 @@ bool ModeBase::deleteRange(Range &r, OperationMode mode, bool addToRegister)
         res = doc()->removeText(r.toEditorRange(), mode == Block);
     }
 
-    QChar chosenRegister = getChosenRegister(ZeroRegister);
+    // the BlackHoleRegister here is only a placeholder to signify that no register was selected
+    // this is needed because the fallback register depends on whether the deleted text spans a line/lines
+    QChar chosenRegister = getChosenRegister(BlackHoleRegister);
     if (addToRegister) {
-        if (r.startLine == r.endLine) {
-            chosenRegister = getChosenRegister(SmallDeleteRegister);
-            fillRegister(chosenRegister, removedText, mode);
-        } else {
-            fillRegister(chosenRegister,  removedText, mode);
-        }
+        fillRegister(chosenRegister, removedText, mode);
+    }
+
+    const QChar lastChar = removedText.count() > 0 ? removedText.back() : QLatin1Char('\0');
+    if (r.startLine != r.endLine || lastChar == QLatin1Char('\n') || lastChar == QLatin1Char('\r')) {
+        // for deletes spanning a line/lines, always prepend to the numbered registers
+        fillRegister(PrependNumberedRegister, removedText, mode);
+        chosenRegister = PrependNumberedRegister;
+    } else if (chosenRegister == BlackHoleRegister) {
+        // only set the SmallDeleteRegister when no register was selected
+        fillRegister(SmallDeleteRegister, removedText, mode);
+        chosenRegister = SmallDeleteRegister;
     }
     yankToClipBoard(chosenRegister, removedText);
 
@@ -145,7 +137,6 @@ const QChar ModeBase::getCharUnderCursor() const
 
 const QString ModeBase::getWordUnderCursor() const
 {
-
     return doc()->text(getWordRangeUnderCursor());
 }
 
@@ -156,9 +147,7 @@ const KTextEditor::Range ModeBase::getWordRangeUnderCursor() const
     // find first character that is a “word letter” and start the search there
     QChar ch = doc()->characterAt(c);
     int i = 0;
-    while (!ch.isLetterOrNumber() && ! ch.isMark() && ch != QLatin1Char('_')
-            && m_extraWordCharacters.indexOf(ch) == -1) {
-
+    while (!ch.isLetterOrNumber() && !ch.isMark() && ch != QLatin1Char('_') && m_extraWordCharacters.indexOf(ch) == -1) {
         // advance cursor one position
         c.setColumn(c.column() + 1);
         if (c.column() > doc()->lineLength(c.line())) {
@@ -195,9 +184,9 @@ KTextEditor::Cursor ModeBase::findNextWordStart(int fromLine, int fromColumn, bo
     }
     startOfWordPattern.append(QLatin1Char(')'));
 
-    QRegExp startOfWord(startOfWordPattern);      // start of a word
-    QRegExp nonSpaceAfterSpace(QLatin1String("\\s\\S"));       // non-space right after space
-    QRegExp nonWordAfterWord(QLatin1String("\\b(?!\\s)\\W"));  // word-boundary followed by a non-word which is not a space
+    const QRegularExpression startOfWord(startOfWordPattern, QRegularExpression::UseUnicodePropertiesOption); // start of a word
+    static const QRegularExpression nonSpaceAfterSpace(QStringLiteral("\\s\\S"), QRegularExpression::UseUnicodePropertiesOption); // non-space right after space
+    static const QRegularExpression nonWordAfterWord(QStringLiteral("\\b(?!\\s)\\W"), QRegularExpression::UseUnicodePropertiesOption); // word-boundary followed by a non-word which is not a space
 
     int l = fromLine;
     int c = fromColumn;
@@ -205,9 +194,9 @@ KTextEditor::Cursor ModeBase::findNextWordStart(int fromLine, int fromColumn, bo
     bool found = false;
 
     while (!found) {
-        int c1 = startOfWord.indexIn(line, c + 1);
-        int c2 = nonSpaceAfterSpace.indexIn(line, c);
-        int c3 = nonWordAfterWord.indexIn(line, c + 1);
+        int c1 = line.indexOf(startOfWord, c + 1);
+        int c2 = line.indexOf(nonSpaceAfterSpace, c);
+        int c3 = line.indexOf(nonWordAfterWord, c + 1);
 
         if (c1 == -1 && c2 == -1 && c3 == -1) {
             if (onlyCurrentLine) {
@@ -257,10 +246,10 @@ KTextEditor::Cursor ModeBase::findNextWORDStart(int fromLine, int fromColumn, bo
     int c = fromColumn;
 
     bool found = false;
-    QRegExp startOfWORD(QLatin1String("\\s\\S"));
+    static const QRegularExpression startOfWORD(QStringLiteral("\\s\\S"), QRegularExpression::UseUnicodePropertiesOption);
 
     while (!found) {
-        c = startOfWORD.indexIn(line, c);
+        c = line.indexOf(startOfWORD, c);
 
         if (c == -1) {
             if (onlyCurrentLine) {
@@ -293,13 +282,13 @@ KTextEditor::Cursor ModeBase::findPrevWordEnd(int fromLine, int fromColumn, bool
 {
     QString line = getLine(fromLine);
 
-    QString endOfWordPattern = QStringLiteral("\\S\\s|\\S$|\\w\\W|\\S\\b|^$");
+    QString endOfWordPattern = QStringLiteral("\\S\\s|\\S$|\\S\\b|\\w\\W|^$");
 
     if (m_extraWordCharacters.length() > 0) {
         endOfWordPattern.append(QLatin1String("|[") + m_extraWordCharacters + QLatin1String("][^") + m_extraWordCharacters + QLatin1Char(']'));
     }
 
-    QRegExp endOfWord(endOfWordPattern);
+    const QRegularExpression endOfWord(endOfWordPattern);
 
     int l = fromLine;
     int c = fromColumn;
@@ -307,7 +296,7 @@ KTextEditor::Cursor ModeBase::findPrevWordEnd(int fromLine, int fromColumn, bool
     bool found = false;
 
     while (!found) {
-        int c1 = endOfWord.lastIndexIn(line, c - 1);
+        int c1 = line.lastIndexOf(endOfWord, c - 1);
 
         if (c1 != -1 && c - 1 != -1) {
             found = true;
@@ -333,9 +322,7 @@ KTextEditor::Cursor ModeBase::findPrevWORDEnd(int fromLine, int fromColumn, bool
 {
     QString line = getLine(fromLine);
 
-    QRegExp endOfWORDPattern(QLatin1String("\\S\\s|\\S$|^$"));
-
-    QRegExp endOfWORD(endOfWORDPattern);
+    static const QRegularExpression endOfWORDPattern(QStringLiteral("\\S\\s|\\S$|^$"), QRegularExpression::UseUnicodePropertiesOption);
 
     int l = fromLine;
     int c = fromColumn;
@@ -343,7 +330,7 @@ KTextEditor::Cursor ModeBase::findPrevWORDEnd(int fromLine, int fromColumn, bool
     bool found = false;
 
     while (!found) {
-        int c1 = endOfWORD.lastIndexIn(line, c - 1);
+        int c1 = line.lastIndexOf(endOfWORDPattern, c - 1);
 
         if (c1 != -1 && c - 1 != -1) {
             found = true;
@@ -377,10 +364,10 @@ KTextEditor::Cursor ModeBase::findPrevWordStart(int fromLine, int fromColumn, bo
     }
     startOfWordPattern.append(QLatin1Char(')'));
 
-    QRegExp startOfWord(startOfWordPattern);      // start of a word
-    QRegExp nonSpaceAfterSpace(QLatin1String("\\s\\S"));       // non-space right after space
-    QRegExp nonWordAfterWord(QLatin1String("\\b(?!\\s)\\W"));  // word-boundary followed by a non-word which is not a space
-    QRegExp startOfLine(QLatin1String("^\\S"));                // non-space at start of line
+    const QRegularExpression startOfWord(startOfWordPattern, QRegularExpression::UseUnicodePropertiesOption); // start of a word
+    static const QRegularExpression nonSpaceAfterSpace(QStringLiteral("\\s\\S"), QRegularExpression::UseUnicodePropertiesOption); // non-space right after space
+    static const QRegularExpression nonWordAfterWord(QStringLiteral("\\b(?!\\s)\\W"), QRegularExpression::UseUnicodePropertiesOption); // word-boundary followed by a non-word which is not a space
+    static const QRegularExpression startOfLine(QStringLiteral("^\\S"), QRegularExpression::UseUnicodePropertiesOption); // non-space at start of line
 
     int l = fromLine;
     int c = fromColumn;
@@ -388,10 +375,10 @@ KTextEditor::Cursor ModeBase::findPrevWordStart(int fromLine, int fromColumn, bo
     bool found = false;
 
     while (!found) {
-        int c1 = startOfWord.lastIndexIn(line, -line.length() + c - 1);
-        int c2 = nonSpaceAfterSpace.lastIndexIn(line, -line.length() + c - 2);
-        int c3 = nonWordAfterWord.lastIndexIn(line, -line.length() + c - 1);
-        int c4 = startOfLine.lastIndexIn(line, -line.length() + c - 1);
+        int c1 = (c > 0) ? line.lastIndexOf(startOfWord, c - 1) : -1;
+        int c2 = (c > 1) ? line.lastIndexOf(nonSpaceAfterSpace, c - 2) : -1;
+        int c3 = (c > 0) ? line.lastIndexOf(nonWordAfterWord, c - 1) : -1;
+        int c4 = (c > 0) ? line.lastIndexOf(startOfLine, c - 1) : -1;
 
         if (c1 == -1 && c2 == -1 && c3 == -1 && c4 == -1) {
             if (onlyCurrentLine) {
@@ -438,8 +425,8 @@ KTextEditor::Cursor ModeBase::findPrevWORDStart(int fromLine, int fromColumn, bo
 {
     QString line = getLine(fromLine);
 
-    QRegExp startOfWORD(QLatin1String("\\s\\S"));
-    QRegExp startOfLineWORD(QLatin1String("^\\S"));
+    static const QRegularExpression startOfWORD(QStringLiteral("\\s\\S"), QRegularExpression::UseUnicodePropertiesOption);
+    static const QRegularExpression startOfLineWORD(QStringLiteral("^\\S"), QRegularExpression::UseUnicodePropertiesOption);
 
     int l = fromLine;
     int c = fromColumn;
@@ -447,8 +434,8 @@ KTextEditor::Cursor ModeBase::findPrevWORDStart(int fromLine, int fromColumn, bo
     bool found = false;
 
     while (!found) {
-        int c1 = startOfWORD.lastIndexIn(line, -line.length() + c - 2);
-        int c2 = startOfLineWORD.lastIndexIn(line, -line.length() + c - 1);
+        int c1 = (c > 1) ? line.lastIndexOf(startOfWORD, c - 2) : -1;
+        int c2 = (c > 0) ? line.lastIndexOf(startOfLineWORD, c - 1) : -1;
 
         if (c1 == -1 && c2 == -1) {
             if (onlyCurrentLine) {
@@ -492,7 +479,7 @@ KTextEditor::Cursor ModeBase::findWordEnd(int fromLine, int fromColumn, bool onl
         endOfWordPattern.append(QLatin1String("|[") + m_extraWordCharacters + QLatin1String("][^") + m_extraWordCharacters + QLatin1Char(']'));
     }
 
-    QRegExp endOfWORD(endOfWordPattern);
+    const QRegularExpression endOfWORD(endOfWordPattern);
 
     int l = fromLine;
     int c = fromColumn;
@@ -500,7 +487,7 @@ KTextEditor::Cursor ModeBase::findWordEnd(int fromLine, int fromColumn, bool onl
     bool found = false;
 
     while (!found) {
-        int c1 = endOfWORD.indexIn(line, c + 1);
+        int c1 = line.indexOf(endOfWORD, c + 1);
 
         if (c1 != -1) {
             found = true;
@@ -527,7 +514,7 @@ KTextEditor::Cursor ModeBase::findWORDEnd(int fromLine, int fromColumn, bool onl
 {
     QString line = getLine(fromLine);
 
-    QRegExp endOfWORD(QLatin1String("\\S\\s|\\S$"));
+    static const QRegularExpression endOfWORD(QStringLiteral("\\S\\s|\\S$"), QRegularExpression::UseUnicodePropertiesOption);
 
     int l = fromLine;
     int c = fromColumn;
@@ -535,7 +522,7 @@ KTextEditor::Cursor ModeBase::findWORDEnd(int fromLine, int fromColumn, bool onl
     bool found = false;
 
     while (!found) {
-        int c1 = endOfWORD.indexIn(line, c + 1);
+        int c1 = line.indexOf(endOfWORD, c + 1);
 
         if (c1 != -1) {
             found = true;
@@ -586,12 +573,11 @@ Range ModeBase::findSurroundingQuotes(const QChar &c, bool inner) const
 
     // If cursor on the quote we should choose the best direction.
     if (line.at(cursor.column()) == c) {
-
         int attribute = m_view->doc()->kateTextLine(cursor.line())->attribute(cursor.column());
 
         //  If at the beginning of the line - then we might search the end.
-        if (doc()->kateTextLine(cursor.line())->attribute(cursor.column() + 1) == attribute &&
-                doc()->kateTextLine(cursor.line())->attribute(cursor.column() - 1) != attribute) {
+        if (doc()->kateTextLine(cursor.line())->attribute(cursor.column() + 1) == attribute
+            && doc()->kateTextLine(cursor.line())->attribute(cursor.column() - 1) != attribute) {
             r.startColumn = cursor.column();
             r.endColumn = line.indexOf(c, cursor.column() + 1);
 
@@ -599,14 +585,12 @@ Range ModeBase::findSurroundingQuotes(const QChar &c, bool inner) const
         }
 
         //  If at the end of the line - then we might search the beginning.
-        if (doc()->kateTextLine(cursor.line())->attribute(cursor.column() + 1) != attribute &&
-                doc()->kateTextLine(cursor.line())->attribute(cursor.column() - 1) == attribute) {
-
+        if (doc()->kateTextLine(cursor.line())->attribute(cursor.column() + 1) != attribute
+            && doc()->kateTextLine(cursor.line())->attribute(cursor.column() - 1) == attribute) {
             r.startColumn = line.lastIndexOf(c, cursor.column() - 1);
             r.endColumn = cursor.column();
 
             return innerRange(r, inner);
-
         }
         // Try to search the quote to right
         int c1 = line.indexOf(c, cursor.column() + 1);
@@ -621,7 +605,7 @@ Range ModeBase::findSurroundingQuotes(const QChar &c, bool inner) const
         int c2 = line.lastIndexOf(c, cursor.column() - 1);
         if (c2 != -1) {
             r.startColumn = c2;
-            r.endColumn =  cursor.column();
+            r.endColumn = cursor.column();
 
             return innerRange(r, inner);
         }
@@ -640,9 +624,7 @@ Range ModeBase::findSurroundingQuotes(const QChar &c, bool inner) const
     return innerRange(r, inner);
 }
 
-Range ModeBase::findSurroundingBrackets(const QChar &c1, const QChar &c2,
-                                        bool inner, const QChar &nested1,
-                                        const QChar &nested2) const
+Range ModeBase::findSurroundingBrackets(const QChar &c1, const QChar &c2, bool inner, const QChar &nested1, const QChar &nested2) const
 {
     KTextEditor::Cursor cursor(m_view->cursorPosition());
     Range r(cursor, InclusiveMotion);
@@ -732,7 +714,7 @@ Range ModeBase::findSurroundingBrackets(const QChar &c1, const QChar &c2,
     return innerRange(r, inner);
 }
 
-Range ModeBase::findSurrounding(const QRegExp &c1, const QRegExp &c2, bool inner) const
+Range ModeBase::findSurrounding(const QRegularExpression &c1, const QRegularExpression &c2, bool inner) const
 {
     KTextEditor::Cursor cursor(m_view->cursorPosition());
     QString line = getLine();
@@ -805,7 +787,7 @@ QString ModeBase::getRegisterContent(const QChar &reg)
     QString r = m_viInputModeManager->globalState()->registers()->getContent(reg);
 
     if (r.isNull()) {
-        error(i18n("Nothing in register %1", reg));
+        error(i18n("Nothing in register %1", reg.toLower()));
     }
 
     return r;
@@ -965,11 +947,12 @@ Range ModeBase::goVisualLineUpDown(int lines)
         // Adjust for the fact that if the portion of the line before wrapping is indented,
         // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
         const bool isWrappedContinuation = (cache->textLayout(startRealLine, startVisualLine).lineLayout().lineNumber() != 0);
-        const int numInvisibleIndentChars = isWrappedContinuation ? startLine->toVirtualColumn(cache->line(startRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+        const int numInvisibleIndentChars =
+            isWrappedContinuation ? startLine->toVirtualColumn(cache->line(startRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
 
         const int realLineStartColumn = cache->textLayout(startRealLine, startVisualLine).startCol();
         const int lineStartVirtualColumn = startLine->toVirtualColumn(realLineStartColumn, tabstop);
-        const int visualColumnNoInvisibleIndent  = startLine->toVirtualColumn(c.column(), tabstop) - lineStartVirtualColumn;
+        const int visualColumnNoInvisibleIndent = startLine->toVirtualColumn(c.column(), tabstop) - lineStartVirtualColumn;
         m_stickyColumn = visualColumnNoInvisibleIndent + numInvisibleIndentChars;
         Q_ASSERT(m_stickyColumn >= 0);
     }
@@ -981,7 +964,8 @@ Range ModeBase::goVisualLineUpDown(int lines)
     // Adjust for the fact that if the portion of the line before wrapping is indented,
     // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
     const bool isWrappedContinuation = (cache->textLayout(finishRealLine, finishVisualLine).lineLayout().lineNumber() != 0);
-    const int numInvisibleIndentChars = isWrappedContinuation ? endLine->toVirtualColumn(cache->line(finishRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+    const int numInvisibleIndentChars =
+        isWrappedContinuation ? endLine->toVirtualColumn(cache->line(finishRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
     if (m_stickyColumn == (unsigned int)KateVi::EOL) {
         const int visualEndColumn = cache->textLayout(finishRealLine, finishVisualLine).lineLayout().textLength() - 1;
         r.endColumn = endLine->fromVirtualColumn(visualEndColumn + realLineStartColumn - numInvisibleIndentChars, tabstop);
@@ -991,7 +975,8 @@ Range ModeBase::goVisualLineUpDown(int lines)
         int realOffsetToVisualStickyColumn = 0;
         const int lineStartVirtualColumn = endLine->toVirtualColumn(realLineStartColumn, tabstop);
         while (true) {
-            const int visualColumn = endLine->toVirtualColumn(realLineStartColumn + realOffsetToVisualStickyColumn, tabstop) - lineStartVirtualColumn + numInvisibleIndentChars;
+            const int visualColumn =
+                endLine->toVirtualColumn(realLineStartColumn + realOffsetToVisualStickyColumn, tabstop) - lineStartVirtualColumn + numInvisibleIndentChars;
             if (visualColumn >= m_stickyColumn) {
                 break;
             }
@@ -1017,7 +1002,7 @@ bool ModeBase::startNormalMode()
 
     m_viInputModeManager->viEnterNormalMode();
     m_view->doc()->setUndoMergeAllEdits(false);
-    emit m_view->viewModeChanged(m_view, m_view->viewMode());
+    Q_EMIT m_view->viewModeChanged(m_view, m_view->viewMode());
 
     return true;
 }
@@ -1026,7 +1011,7 @@ bool ModeBase::startInsertMode()
 {
     m_viInputModeManager->viEnterInsertMode();
     m_view->doc()->setUndoMergeAllEdits(true);
-    emit m_view->viewModeChanged(m_view, m_view->viewMode());
+    Q_EMIT m_view->viewModeChanged(m_view, m_view->viewMode());
 
     return true;
 }
@@ -1035,7 +1020,7 @@ bool ModeBase::startReplaceMode()
 {
     m_view->doc()->setUndoMergeAllEdits(true);
     m_viInputModeManager->viEnterReplaceMode();
-    emit m_view->viewModeChanged(m_view, m_view->viewMode());
+    Q_EMIT m_view->viewModeChanged(m_view, m_view->viewMode());
 
     return true;
 }
@@ -1052,7 +1037,7 @@ bool ModeBase::startVisualMode()
         m_viInputModeManager->viEnterVisualMode();
     }
 
-    emit m_view->viewModeChanged(m_view, m_view->viewMode());
+    Q_EMIT m_view->viewModeChanged(m_view, m_view->viewMode());
 
     return true;
 }
@@ -1066,7 +1051,7 @@ bool ModeBase::startVisualBlockMode()
         m_viInputModeManager->viEnterVisualMode(ViMode::VisualBlockMode);
     }
 
-    emit m_view->viewModeChanged(m_view, m_view->viewMode());
+    Q_EMIT m_view->viewModeChanged(m_view, m_view->viewMode());
 
     return true;
 }
@@ -1080,7 +1065,7 @@ bool ModeBase::startVisualLineMode()
         m_viInputModeManager->viEnterVisualMode(ViMode::VisualLineMode);
     }
 
-    emit m_view->viewModeChanged(m_view, m_view->viewMode());
+    Q_EMIT m_view->viewModeChanged(m_view, m_view->viewMode());
 
     return true;
 }
@@ -1114,8 +1099,7 @@ QString ModeBase::getVerbatimKeys() const
     return m_keysVerbatim;
 }
 
-const QChar ModeBase::getCharAtVirtualColumn(const QString &line, int virtualColumn,
-        int tabWidth) const
+const QChar ModeBase::getCharAtVirtualColumn(const QString &line, int virtualColumn, int tabWidth) const
 {
     int column = 0;
     int tempCol = 0;
@@ -1157,28 +1141,29 @@ void ModeBase::addToNumberUnderCursor(int count)
         return;
     }
 
-    int numberStartPos = -1;
-    QString numberAsString;
-    QRegExp numberRegex(QLatin1String("(0x)([0-9a-fA-F]+)|\\-?\\d+"));
     const int cursorColumn = c.column();
-    const int currentLineLength = doc()->lineLength(c.line());
-    const KTextEditor::Cursor prevWordStart = findPrevWordStart(c.line(), cursorColumn);
+    const int cursorLine = c.line();
+    const KTextEditor::Cursor prevWordStart = findPrevWordStart(cursorLine, cursorColumn);
     int wordStartPos = prevWordStart.column();
-    if (prevWordStart.line() < c.line()) {
+    if (prevWordStart.line() < cursorLine) {
         // The previous word starts on the previous line: ignore.
         wordStartPos = 0;
     }
     if (wordStartPos > 0 && line.at(wordStartPos - 1) == QLatin1Char('-')) {
         wordStartPos--;
     }
-    for (int searchFromColumn = wordStartPos; searchFromColumn < currentLineLength; searchFromColumn++) {
-        numberStartPos = numberRegex.indexIn(line, searchFromColumn);
 
-        numberAsString = numberRegex.cap();
-
-        const bool numberEndedBeforeCursor = (numberStartPos + numberAsString.length() <= c.column());
+    int numberStartPos = -1;
+    QString numberAsString;
+    static const QRegularExpression numberRegex(QStringLiteral("0x[0-9a-fA-F]+|\\-?\\d+"));
+    auto numberMatchIter = numberRegex.globalMatch(line, wordStartPos);
+    while (numberMatchIter.hasNext()) {
+        const auto numberMatch = numberMatchIter.next();
+        const bool numberEndedBeforeCursor = (numberMatch.capturedStart() + numberMatch.capturedLength() <= cursorColumn);
         if (!numberEndedBeforeCursor) {
             // This is the first number-like string under or after the cursor - this'll do!
+            numberStartPos = numberMatch.capturedStart();
+            numberAsString = numberMatch.captured();
             break;
         }
     }
@@ -1189,8 +1174,8 @@ void ModeBase::addToNumberUnderCursor(int count)
     }
 
     bool parsedNumberSuccessfully = false;
-    int base = numberRegex.cap(1).isEmpty() ? 10 : 16;
-    if (base != 16 && numberAsString.startsWith(QLatin1String("0")) && numberAsString.length() > 1) {
+    int base = numberAsString.startsWith(QLatin1String("0x")) ? 16 : 10;
+    if (base != 16 && numberAsString.startsWith(QLatin1Char('0')) && numberAsString.length() > 1) {
         // If a non-hex number with a leading 0 can be parsed as octal, then assume
         // it is octal.
         numberAsString.toInt(&parsedNumberSuccessfully, 8);
@@ -1211,30 +1196,29 @@ void ModeBase::addToNumberUnderCursor(int count)
     } else if (base == 8) {
         basePrefix = QStringLiteral("0");
     }
-    const QString withoutBase = numberAsString.mid(basePrefix.length());
+    const QStringRef withoutBase = numberAsString.midRef(basePrefix.length());
 
     const int newNumber = originalNumber + count;
 
     // Create the new text string to be inserted. Prepend with “0x” if in base 16, and "0" if base 8.
     // For non-decimal numbers, try to keep the length of the number the same (including leading 0's).
-    const QString newNumberPadded = (base == 10) ?
-                                    QStringLiteral("%1").arg(newNumber, 0, base) :
-                                    QStringLiteral("%1").arg(newNumber, withoutBase.length(), base, QLatin1Char('0'));
+    const QString newNumberPadded =
+        (base == 10) ? QStringLiteral("%1").arg(newNumber, 0, base) : QStringLiteral("%1").arg(newNumber, withoutBase.length(), base, QLatin1Char('0'));
     const QString newNumberText = basePrefix + newNumberPadded;
 
     // Replace the old number string with the new.
     doc()->editStart();
-    doc()->removeText(KTextEditor::Range(c.line(), numberStartPos, c.line(), numberStartPos + numberAsString.length()));
-    doc()->insertText(KTextEditor::Cursor(c.line(), numberStartPos), newNumberText);
+    doc()->removeText(KTextEditor::Range(cursorLine, numberStartPos, cursorLine, numberStartPos + numberAsString.length()));
+    doc()->insertText(KTextEditor::Cursor(cursorLine, numberStartPos), newNumberText);
     doc()->editEnd();
     updateCursor(KTextEditor::Cursor(m_view->cursorPosition().line(), numberStartPos + newNumberText.length() - 1));
 }
 
 void ModeBase::switchView(Direction direction)
 {
-
     QList<KTextEditor::ViewPrivate *> visible_views;
-    foreach (KTextEditor::ViewPrivate *view,  KTextEditor::EditorPrivate::self()->views()) {
+    const auto views = KTextEditor::EditorPrivate::self()->views();
+    for (KTextEditor::ViewPrivate *view : views) {
         if (view->isVisible()) {
             visible_views.push_back(view);
         }
@@ -1251,7 +1235,7 @@ void ModeBase::switchView(Direction direction)
     int curr_cursor_x = globalPos.x();
 
     KTextEditor::ViewPrivate *bestview = nullptr;
-    int  best_x1 = -1, best_x2 = -1, best_y1 = -1, best_y2 = -1, best_center_y = -1, best_center_x = -1;
+    int best_x1 = -1, best_x2 = -1, best_y1 = -1, best_y2 = -1, best_center_y = -1, best_center_x = -1;
 
     if (direction == Next && visible_views.count() != 1) {
         for (int i = 0; i < visible_views.count(); i++) {
@@ -1264,51 +1248,43 @@ void ModeBase::switchView(Direction direction)
             }
         }
     } else {
-        foreach (KTextEditor::ViewPrivate *view, visible_views) {
+        for (KTextEditor::ViewPrivate *view : qAsConst(visible_views)) {
             QPoint point = view->mapToGlobal(view->pos());
             int x1 = point.x();
             int x2 = point.x() + view->width();
             int y1 = point.y();
             int y2 = point.y() + m_view->height();
-            int  center_y = (y1 + y2) / 2;
-            int  center_x = (x1 + x2) / 2;
+            int center_y = (y1 + y2) / 2;
+            int center_x = (x1 + x2) / 2;
 
             switch (direction) {
             case Left:
-                if (view != m_view && x2 <= curr_x1 &&
-                        (x2 > best_x2 ||
-                         (x2 == best_x2 && qAbs(curr_cursor_y - center_y) < qAbs(curr_cursor_y - best_center_y)) ||
-                         bestview == nullptr)) {
+                if (view != m_view && x2 <= curr_x1
+                    && (x2 > best_x2 || (x2 == best_x2 && qAbs(curr_cursor_y - center_y) < qAbs(curr_cursor_y - best_center_y)) || bestview == nullptr)) {
                     bestview = view;
                     best_x2 = x2;
                     best_center_y = center_y;
                 }
                 break;
             case Right:
-                if (view != m_view && x1 >= curr_x2 &&
-                        (x1 < best_x1 ||
-                         (x1 == best_x1 && qAbs(curr_cursor_y - center_y) < qAbs(curr_cursor_y - best_center_y)) ||
-                         bestview == nullptr)) {
+                if (view != m_view && x1 >= curr_x2
+                    && (x1 < best_x1 || (x1 == best_x1 && qAbs(curr_cursor_y - center_y) < qAbs(curr_cursor_y - best_center_y)) || bestview == nullptr)) {
                     bestview = view;
                     best_x1 = x1;
                     best_center_y = center_y;
                 }
                 break;
             case Down:
-                if (view != m_view && y1 >= curr_y2 &&
-                        (y1 < best_y1 ||
-                         (y1 == best_y1 && qAbs(curr_cursor_x - center_x) < qAbs(curr_cursor_x - best_center_x)) ||
-                         bestview == nullptr)) {
+                if (view != m_view && y1 >= curr_y2
+                    && (y1 < best_y1 || (y1 == best_y1 && qAbs(curr_cursor_x - center_x) < qAbs(curr_cursor_x - best_center_x)) || bestview == nullptr)) {
                     bestview = view;
                     best_y1 = y1;
                     best_center_x = center_x;
                 }
                 break;
             case Up:
-                if (view != m_view && y2 <= curr_y1 &&
-                        (y2 > best_y2 ||
-                         (y2 == best_y2 && qAbs(curr_cursor_x - center_x) < qAbs(curr_cursor_x - best_center_x)) ||
-                         bestview == nullptr)) {
+                if (view != m_view && y2 <= curr_y1
+                    && (y2 > best_y2 || (y2 == best_y2 && qAbs(curr_cursor_x - center_x) < qAbs(curr_cursor_x - best_center_x)) || bestview == nullptr)) {
                     bestview = view;
                     best_y2 = y2;
                     best_center_x = center_x;
@@ -1317,7 +1293,6 @@ void ModeBase::switchView(Direction direction)
             default:
                 return;
             }
-
         }
     }
     if (bestview != nullptr) {
@@ -1328,12 +1303,24 @@ void ModeBase::switchView(Direction direction)
 
 Range ModeBase::motionFindPrev()
 {
-    return m_viInputModeManager->searcher()->motionFindPrev(getCount());
+    Searcher *searcher = m_viInputModeManager->searcher();
+    Range match = searcher->motionFindPrev(getCount());
+    if (searcher->lastSearchWrapped()) {
+        m_view->showSearchWrappedHint(/*isReverseSearch*/ true);
+    }
+
+    return match;
 }
 
 Range ModeBase::motionFindNext()
 {
-    return m_viInputModeManager->searcher()->motionFindNext(getCount());
+    Searcher *searcher = m_viInputModeManager->searcher();
+    Range match = searcher->motionFindNext(getCount());
+    if (searcher->lastSearchWrapped()) {
+        m_view->showSearchWrappedHint(/*isReverseSearch*/ false);
+    }
+
+    return match;
 }
 
 void ModeBase::goToPos(const Range &r)
