@@ -13,8 +13,6 @@
 #include <vimode/inputmodemanager.h>
 #include <vimode/modes/modebase.h>
 
-#include <KLocalizedString>
-
 using namespace KateVi;
 
 Searcher::Searcher(InputModeManager *manager)
@@ -23,10 +21,6 @@ Searcher::Searcher(InputModeManager *manager)
 {
 }
 
-Searcher::~Searcher()
-{
-}
-
 const QString Searcher::getLastSearchPattern() const
 {
     return m_lastSearchConfig.pattern;
@@ -37,6 +31,11 @@ void Searcher::setLastSearchParams(const SearchParams &searchParams)
     m_lastSearchConfig = searchParams;
 }
 
+bool Searcher::lastSearchWrapped() const
+{
+    return m_lastSearchWrapped;
+}
+
 void Searcher::findNext()
 {
     const Range r = motionFindPrev();
@@ -81,7 +80,7 @@ Range Searcher::motionFindPrev(int count)
     return Range(match.endLine, match.endColumn - 1, ExclusiveMotion);
 }
 
-Range Searcher::findPatternForMotion(const SearchParams &searchParams, const KTextEditor::Cursor &startFrom, int count) const
+Range Searcher::findPatternForMotion(const SearchParams &searchParams, const KTextEditor::Cursor &startFrom, int count)
 {
     if (searchParams.pattern.isEmpty()) {
         return Range::invalid();
@@ -114,10 +113,11 @@ KTextEditor::Range Searcher::findPattern(const SearchParams &searchParams, const
     return findPatternWorker(searchParams, startFrom, count);
 }
 
-KTextEditor::Range Searcher::findPatternWorker(const SearchParams &searchParams, const KTextEditor::Cursor &startFrom, int count) const
+KTextEditor::Range Searcher::findPatternWorker(const SearchParams &searchParams, const KTextEditor::Cursor &startFrom, int count)
 {
     KTextEditor::Cursor searchBegin = startFrom;
     KTextEditor::SearchOptions flags = KTextEditor::Regex;
+    m_lastSearchWrapped = false;
 
     const QString &pattern = searchParams.pattern;
 
@@ -128,7 +128,6 @@ KTextEditor::Range Searcher::findPatternWorker(const SearchParams &searchParams,
         flags |= KTextEditor::CaseInsensitive;
     }
     KTextEditor::Range finalMatch;
-    bool searchWrapped = false;
     for (int i = 0; i < count; i++) {
         if (!searchParams.isBackwards) {
             const KTextEditor::Range matchRange =
@@ -146,7 +145,7 @@ KTextEditor::Range Searcher::findPatternWorker(const SearchParams &searchParams,
                     m_view->doc()->searchText(KTextEditor::Range(m_view->doc()->documentRange().start(), m_view->doc()->documentEnd()), pattern, flags).first();
                 if (wrappedMatchRange.isValid()) {
                     finalMatch = wrappedMatchRange;
-                    searchWrapped = true;
+                    m_lastSearchWrapped = true;
                 } else {
                     return KTextEditor::Range::invalid();
                 }
@@ -202,7 +201,7 @@ KTextEditor::Range Searcher::findPatternWorker(const SearchParams &searchParams,
 
                 if (wrappedMatchRange.isValid()) {
                     finalMatch = wrappedMatchRange;
-                    searchWrapped = true;
+                    m_lastSearchWrapped = true;
                 } else {
                     return KTextEditor::Range::invalid();
                 }
@@ -210,10 +209,5 @@ KTextEditor::Range Searcher::findPatternWorker(const SearchParams &searchParams,
         }
         searchBegin = finalMatch.start();
     }
-
-    if (searchWrapped) {
-        m_viInputModeManager->getCurrentViModeHandler()->message(i18n("Search wrapped"));
-    }
-
     return finalMatch;
 }
